@@ -12,6 +12,7 @@ from .obstruction_inferral import (
 from .obstruction_transitivity import ObstructionTransitivity
 from .requirement_placement import RequirementPlacement
 from .row_col_separation import RowColSeparation
+from .subclass_verification import SubclassVerificationAlgorithm
 
 __all__ = [
     "DatabaseEnumeration",
@@ -31,4 +32,5 @@ __all__ = [
     "GriddedPermReduction",
     "RequirementPlacement",
     "RowColSeparation",
+    "SubclassVerificationAlgorithm",
 ]
