@@ -5,10 +5,10 @@ The implementation of the fusion algorithm
 """
 from collections import Counter
 from itertools import chain
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Optional
 
-from permuta import Perm
 from tilings.assumptions import (
+    ComponentAssumption,
     SkewComponentAssumption,
     SumComponentAssumption,
     TrackingAssumption,
@@ -30,9 +30,22 @@ class Fusion:
 
     If incited `col_ids` is provided it attempts to fuse column `col_idx` with
     column `col_idx+1`.
+
+    Isolation Levels:
+      None:             no restrictions
+      "noninteracting": there can be at most one assumption involving the cells in the
+                          fused rows/cols
+      "isolated":       there can be no assumptions except the one induced by the fusion
     """
 
-    def __init__(self, tiling, row_idx=None, col_idx=None, tracked: bool = False):
+    def __init__(
+        self,
+        tiling,
+        row_idx=None,
+        col_idx=None,
+        tracked: bool = False,
+        isolation_level: Optional[str] = None,
+    ):
         self._tiling: "Tiling" = tiling
         self._assumptions_fuse_counters = None
         self._obstruction_fuse_counter = None
@@ -48,6 +61,13 @@ class Fusion:
             self._fuse_row = True
         else:
             raise RuntimeError("Cannot specify a row and a columns")
+        self.isolation_level = isolation_level
+        assert self.isolation_level in [
+            None,
+            "noninteracting",
+            "isolated",
+        ], "The only valid isolation levels are None, 'noninteracting', and 'isolated'."
+        self._fused_tiling: Optional["Tiling"] = None
 
     def _fuse_gridded_perm(self, gp):
         """
@@ -126,11 +146,11 @@ class Fusion:
         left, right = [], []
         for (x, y) in self._tiling.active_cells:
             if self._fuse_row and y == self._row_idx:
-                left.append(GriddedPerm.single_cell(Perm((0,)), (x, y)))
-                right.append(GriddedPerm.single_cell(Perm((0,)), (x, y + 1)))
+                left.append(GriddedPerm.single_cell((0,), (x, y)))
+                right.append(GriddedPerm.single_cell((0,), (x, y + 1)))
             if not self._fuse_row and x == self._col_idx:
-                left.append(GriddedPerm.single_cell(Perm((0,)), (x, y)))
-                right.append(GriddedPerm.single_cell(Perm((0,)), (x + 1, y)))
+                left.append(GriddedPerm.single_cell((0,), (x, y)))
+                right.append(GriddedPerm.single_cell((0,), (x + 1, y)))
         return tuple(sorted(left)), tuple(sorted(right))
 
     def new_positive_requirement(self):
@@ -145,14 +165,14 @@ class Fusion:
             res = []
             for idx, c1 in enumerate(cells):
                 for c2 in cells[idx:]:
-                    res.append(GriddedPerm(Perm((0, 1)), (c1, c2)))
+                    res.append(GriddedPerm((0, 1), (c1, c2)))
                     if self._fuse_row:
-                        res.append(GriddedPerm(Perm((1, 0)), (c1, c2)))
+                        res.append(GriddedPerm((1, 0), (c1, c2)))
                     else:
-                        res.append(GriddedPerm(Perm((1, 0)), (c2, c1)))
+                        res.append(GriddedPerm((1, 0), (c2, c1)))
             return sorted(res)
         if self._positive_left or self._positive_right:
-            return sorted(GriddedPerm.single_cell(Perm((0,)), cell) for cell in cells)
+            return sorted(GriddedPerm.single_cell((0,), cell) for cell in cells)
         raise ValueError("no positive left right requirement")
 
     def is_positive_left_or_right_requirement(self, requirement):
@@ -232,6 +252,10 @@ class Fusion:
         are all contained entirely on the left of the fusion region, entirely
         on the right, or split in every possible way.
         """
+        if isinstance(assumption, ComponentAssumption):
+            return self.is_left_sided_assumption(
+                assumption
+            ) and self.is_right_sided_assumption(assumption)
         return self._can_fuse_set_of_gridded_perms(fuse_counter) or (
             all(count == 1 for gp, count in fuse_counter.items())
             and self._is_one_sided_assumption(assumption)
@@ -267,12 +291,47 @@ class Fusion:
         Return the assumption that needs to counted in order to enumerate.
         """
         return TrackingAssumption(
-            GriddedPerm.single_cell(Perm((0,)), cell)
+            GriddedPerm.single_cell((0,), cell)
             for cell in self._tiling.active_cells
             if (self._fuse_row and cell[1] == self._row_idx)
             or (not self._fuse_row and cell[0] == self._col_idx)
         )
 
+    def _num_fusing_assumptions(self):
+        if self._fuse_row:
+            fusing_cells = [
+                (i, self._row_idx) for i in range(self._tiling.dimensions[0])
+            ] + [(i, self._row_idx + 1) for i in range(self._tiling.dimensions[0])]
+        else:
+            fusing_cells = [
+                (self._col_idx, i) for i in range(self._tiling.dimensions[1])
+            ] + [(self._col_idx + 1, i) for i in range(self._tiling.dimensions[1])]
+        return len(
+            [
+                assumption
+                for assumption in self._tiling.assumptions
+                if any(cell in gp.pos for gp in assumption.gps for cell in fusing_cells)
+            ]
+        )
+
+    def _check_isolation_level(self):
+        """
+        Checks whether the requirements for self.isolation_level are met.
+        """
+        if self.isolation_level is None:
+            return True
+
+        if self.isolation_level == "noninteracting":
+            return self._num_fusing_assumptions() <= 1
+
+        if self.isolation_level == "isolated":
+            return len(self._tiling.assumptions) == 0 or (
+                len(self._tiling.assumptions) == 1
+                and self._num_fusing_assumptions() == 1
+            )
+
+        assert False, "{} is an invalid isolation_level".format(self.isolation_level)
+
     def fusable(self):
         """
         Check if the fusion is possible.
@@ -288,29 +347,39 @@ class Fusion:
                 self._tiling.assumptions, self.assumptions_fuse_counters
             )
         )
-        return obs_fusable and req_fusable and ass_fusable
+        return (
+            obs_fusable
+            and req_fusable
+            and ass_fusable
+            and self._check_isolation_level()
+            and not self.fused_tiling()
+            .add_list_requirement(list(self.new_assumption().gps))
+            .is_empty()
+        )
 
     def fused_tiling(self) -> "Tiling":
         """
         Return the fused tiling.
         """
-        assumptions = [
-            ass.__class__(gps)
-            for ass, gps in zip(
-                self._tiling.assumptions, self.assumptions_fuse_counters
+        if self._fused_tiling is None:
+            assumptions = [
+                ass.__class__(gps)
+                for ass, gps in zip(
+                    self._tiling.assumptions, self.assumptions_fuse_counters
+                )
+            ]
+            if self._tracked:
+                assumptions.append(self.new_assumption())
+            requirements = self.requirements_fuse_counters
+            if self._positive_left or self._positive_right:
+                new_positive_requirement = self.new_positive_requirement()
+                requirements = requirements + [new_positive_requirement]
+            self._fused_tiling = self._tiling.__class__(
+                obstructions=self.obstruction_fuse_counter.keys(),
+                requirements=requirements,
+                assumptions=assumptions,
             )
-        ]
-        if self._tracked:
-            assumptions.append(self.new_assumption())
-        requirements = self.requirements_fuse_counters
-        if self._positive_left or self._positive_right:
-            new_positive_requirement = self.new_positive_requirement()
-            requirements = requirements + [new_positive_requirement]
-        return self._tiling.__class__(
-            obstructions=self.obstruction_fuse_counter.keys(),
-            requirements=requirements,
-            assumptions=assumptions,
-        )
+        return self._fused_tiling
 
 
 class ComponentFusion(Fusion):
@@ -328,12 +397,26 @@ class ComponentFusion(Fusion):
     column `col_idx+1`.
     """
 
-    def __init__(self, tiling, *, row_idx=None, col_idx=None, tracked: bool = False):
+    def __init__(
+        self,
+        tiling,
+        *,
+        row_idx=None,
+        col_idx=None,
+        tracked: bool = False,
+        isolation_level: Optional[str] = None
+    ):
         if tiling.requirements:
             raise NotImplementedError(
                 "Component fusion does not handle " "requirements at the moment"
             )
-        super().__init__(tiling, row_idx=row_idx, col_idx=col_idx, tracked=tracked)
+        super().__init__(
+            tiling,
+            row_idx=row_idx,
+            col_idx=col_idx,
+            tracked=tracked,
+            isolation_level=isolation_level,
+        )
         self._first_cell = None
         self._second_cell = None
 
@@ -413,13 +496,13 @@ class ComponentFusion(Fusion):
         scell = self.second_cell
         if self._fuse_row:
             possible_obs = [
-                GriddedPerm(Perm((0, 1)), (fcell, scell)),
-                GriddedPerm(Perm((1, 0)), (scell, fcell)),
+                GriddedPerm((0, 1), (fcell, scell)),
+                GriddedPerm((1, 0), (scell, fcell)),
             ]
         else:
             possible_obs = [
-                GriddedPerm(Perm((0, 1)), (fcell, scell)),
-                GriddedPerm(Perm((1, 0)), (fcell, scell)),
+                GriddedPerm((0, 1), (fcell, scell)),
+                GriddedPerm((1, 0), (fcell, scell)),
             ]
         return any(ob in possible_obs for ob in self._tiling.obstructions)
 
@@ -459,6 +542,21 @@ class ComponentFusion(Fusion):
             self._unfuse_gridded_perm(ob) for ob in self.obstruction_fuse_counter
         )
 
+    def _can_fuse_assumption(self, assumption, fuse_counter):
+        """
+        Return True if an assumption can be fused. That is, prefusion, the gps
+        are all contained entirely on the left of the fusion region, entirely
+        on the right, or split in every possible way.
+        """
+        if not isinstance(assumption, ComponentAssumption):
+            return self.is_left_sided_assumption(
+                assumption
+            ) and self.is_right_sided_assumption(assumption)
+        return self._can_fuse_set_of_gridded_perms(fuse_counter) or (
+            all(count == 1 for gp, count in fuse_counter.items())
+            and self._is_one_sided_assumption(assumption)
+        )
+
     def _can_fuse_set_of_gridded_perms(self, fuse_counter):
         raise NotImplementedError
 
@@ -473,7 +571,8 @@ class ComponentFusion(Fusion):
         if not self._pre_check() or not self.has_crossing_len2_ob():
             return False
         new_tiling = self._tiling.add_obstructions(self.obstructions_to_add())
-        return self._tiling == new_tiling
+
+        return self._tiling == new_tiling and self._check_isolation_level()
 
     def new_assumption(self):
         """
@@ -481,11 +580,11 @@ class ComponentFusion(Fusion):
         """
         fcell = self.first_cell
         scell = self.second_cell
-        gps = (GriddedPerm.single_cell(Perm((0,)), fcell),)
+        gps = (GriddedPerm.single_cell((0,), fcell),)
         if self._fuse_row:
-            sum_ob = GriddedPerm(Perm((1, 0)), (scell, fcell))
+            sum_ob = GriddedPerm((1, 0), (scell, fcell))
         else:
-            sum_ob = GriddedPerm(Perm((1, 0)), (fcell, scell))
+            sum_ob = GriddedPerm((1, 0), (fcell, scell))
         if sum_ob in self._tiling.obstructions:
             return SumComponentAssumption(gps)
         return SkewComponentAssumption(gps)
