@@ -1,13 +1,23 @@
-from typing import Iterable, Optional
+from typing import TYPE_CHECKING, Iterable, List, Optional, Union
 
 from logzero import logger
 
 from comb_spec_searcher import StrategyPack
-from comb_spec_searcher.strategies import AbstractStrategy
+from comb_spec_searcher.strategies import (
+    AbstractStrategy,
+    Strategy,
+    StrategyFactory,
+    VerificationStrategy,
+)
 from permuta import Perm
 from permuta.misc import DIR_EAST, DIR_NORTH, DIR_SOUTH, DIR_WEST, DIRS
 from tilings import strategies as strat
 
+if TYPE_CHECKING:
+    from tilings import Tiling
+
+CSSstrategy = Union[Strategy, StrategyFactory, VerificationStrategy]
+
 
 class TileScopePack(StrategyPack):
     # Method to add power to a pack
@@ -38,6 +48,66 @@ class TileScopePack(StrategyPack):
             iterative=self.iterative,
         )
 
+    def setup_subclass_verification(self, start_tiling: "Tiling") -> "TileScopePack":
+        """
+        If the subclass verification strategy already has a list of perms to check,
+        we leave it alone. Otherwise we:
+         - compute the cell basis for each cell
+         - consider the permutations avoiding some cell basis
+         - add to the list of perms to check any perm that avoids some cell basis and
+           has length strictly smaller than the maximum length cell basis element.
+        """
+
+        def replace_list(strats):
+            """
+            Find subclass verification and alter its perms_to_check variable.
+            """
+            res = []
+            for strategy in strats:
+                if isinstance(strategy, strat.SubclassVerificationFactory):
+                    printed_log = False
+                    if strategy.perms_to_check is None:
+                        new_perms_to_check = set()
+                        cell_bases = set(
+                            tuple(obs) for obs, _ in start_tiling.cell_basis().values()
+                        )
+                        max_length = (
+                            max(max(len(b) for b in basis) for basis in cell_bases) - 1
+                        )
+                        perm_gen = Perm.up_to_length(max_length)
+                        for perm in perm_gen:
+                            if len(perm) == 0:
+                                continue
+                            if any(perm.avoids(*basis) for basis in cell_bases):
+                                new_perms_to_check.add(perm)
+                        res.append(strategy.change_perms(new_perms_to_check))
+                        if start_tiling.dimensions == (1, 1):
+                            logger.info(
+                                "SubclassVerification set up to check the proper "
+                                "principal subclasses of Av(%s)",
+                                ", ".join(map(str, cell_bases.pop())),
+                            )
+                            printed_log = True
+                    if not printed_log:
+                        logger.info(
+                            "SubclassVerification set up to check the subclasses: "
+                            "Av(%s)",
+                            "), Av(".join(map(str, strategy.perms_to_check)),
+                        )
+                else:
+                    res.append(strategy)
+            return res
+
+        return self.__class__(
+            ver_strats=replace_list(self.ver_strats),
+            inferral_strats=replace_list(self.inferral_strats),
+            initial_strats=replace_list(self.initial_strats),
+            expansion_strats=list(map(replace_list, self.expansion_strats)),
+            name=self.name,
+            symmetries=self.symmetries,
+            iterative=self.iterative,
+        )
+
     def make_tracked(self, interleaving: str = "none"):
         """Add assumption tracking strategies."""
         pack = self
@@ -46,7 +116,11 @@ class TileScopePack(StrategyPack):
         return pack
 
     def make_fusion(
-        self, component: bool = False, tracked: bool = True, apply_first: bool = False
+        self,
+        component: bool = False,
+        tracked: bool = True,
+        apply_first: bool = False,
+        isolation_level: Optional[str] = None,
     ) -> "TileScopePack":
         """
         Create a new pack by adding fusion to the current pack.
@@ -60,17 +134,25 @@ class TileScopePack(StrategyPack):
             pack = pack.make_tracked()
             if component:
                 pack = pack.add_initial(
-                    strat.SplittingStrategy(ignore_parent=True), apply_first=True
+                    strat.DetectComponentsStrategy(ignore_parent=True), apply_first=True
                 )
         if component:
             pack = pack.add_initial(
-                strat.ComponentFusionFactory(tracked=tracked),
-                "component_fusion",
+                strat.ComponentFusionFactory(
+                    tracked=tracked, isolation_level=isolation_level
+                ),
+                "component_fusion{}".format(
+                    "" if isolation_level is None else "_" + isolation_level
+                ),
                 apply_first=apply_first,
             )
         else:
             pack = pack.add_initial(
-                strat.FusionFactory(tracked=tracked), "fusion", apply_first=apply_first
+                strat.FusionFactory(tracked=tracked, isolation_level=isolation_level),
+                "fusion{}".format(
+                    "" if isolation_level is None else "_" + isolation_level
+                ),
+                apply_first=apply_first,
             )
         return pack
 
@@ -150,39 +232,12 @@ class TileScopePack(StrategyPack):
     # Creation of the base pack
     @classmethod
     def all_the_strategies(cls, length: int = 1) -> "TileScopePack":
-        return TileScopePack(
-            initial_strats=[
-                strat.FactorFactory(unions=False),
-                strat.RequirementCorroborationFactory(),
-            ],
-            ver_strats=[
-                strat.BasicVerificationStrategy(),
-                strat.InsertionEncodingVerificationStrategy(),
-                strat.OneByOneVerificationStrategy(),
-                strat.LocallyFactorableVerificationStrategy(),
-            ],
-            inferral_strats=[
-                strat.RowColumnSeparationStrategy(),
-                strat.ObstructionTransitivityFactory(),
-            ],
-            expansion_strats=[
-                [strat.RequirementInsertionFactory(maxreqlen=length)],
-                [strat.AllPlacementsFactory()],
-            ],
-            name="all_the_strategies",
-        )
+        initial_strats: List[CSSstrategy] = [strat.FactorFactory()]
+        if length > 1:
+            initial_strats.append(strat.RequirementCorroborationFactory())
 
-    @classmethod
-    def pattern_placements(
-        cls, length: int = 1, partial: bool = False,
-    ) -> "TileScopePack":
-        name = "{}{}{}_placements".format(
-            "length_{}_".format(length) if length > 1 else "",
-            "partial_" if partial else "",
-            "pattern" if length > 1 else "point",
-        )
         return TileScopePack(
-            initial_strats=[strat.PatternPlacementFactory(partial=partial)],
+            initial_strats=initial_strats,
             ver_strats=[
                 strat.BasicVerificationStrategy(),
                 strat.InsertionEncodingVerificationStrategy(),
@@ -195,11 +250,43 @@ class TileScopePack(StrategyPack):
             ],
             expansion_strats=[
                 [
-                    strat.FactorFactory(unions=True),
-                    strat.CellInsertionFactory(maxreqlen=length),
+                    strat.RequirementInsertionFactory(maxreqlen=length),
+                    strat.AllPlacementsFactory(),
                 ],
-                [strat.RequirementCorroborationFactory()],
             ],
+            name="all_the_strategies",
+        )
+
+    @classmethod
+    def pattern_placements(
+        cls, length: int = 1, partial: bool = False
+    ) -> "TileScopePack":
+        name = "{}{}{}_placements".format(
+            "length_{}_".format(length) if length > 1 else "",
+            "partial_" if partial else "",
+            "pattern" if length > 1 else "pattern_point",
+        )
+
+        expansion_strats: List[CSSstrategy] = [
+            strat.FactorFactory(unions=True),
+            strat.CellInsertionFactory(maxreqlen=length),
+        ]
+        if length > 1:
+            expansion_strats.append(strat.RequirementCorroborationFactory())
+
+        return TileScopePack(
+            initial_strats=[strat.PatternPlacementFactory(partial=partial)],
+            ver_strats=[
+                strat.BasicVerificationStrategy(),
+                strat.InsertionEncodingVerificationStrategy(),
+                strat.OneByOneVerificationStrategy(),
+                strat.LocallyFactorableVerificationStrategy(),
+            ],
+            inferral_strats=[
+                strat.RowColumnSeparationStrategy(),
+                strat.ObstructionTransitivityFactory(),
+            ],
+            expansion_strats=[expansion_strats],
             name=name,
         )
 
@@ -211,11 +298,13 @@ class TileScopePack(StrategyPack):
             "length_{}_".format(length) if length > 1 else "",
             "partial_" if partial else "",
         )
+
+        initial_strats: List[CSSstrategy] = [strat.FactorFactory()]
+        if length > 1:
+            initial_strats.append(strat.RequirementCorroborationFactory())
+
         return TileScopePack(
-            initial_strats=[
-                strat.FactorFactory(),
-                strat.RequirementCorroborationFactory(),
-            ],
+            initial_strats=initial_strats,
             ver_strats=[
                 strat.BasicVerificationStrategy(),
                 strat.InsertionEncodingVerificationStrategy(),
@@ -227,26 +316,25 @@ class TileScopePack(StrategyPack):
                 strat.ObstructionTransitivityFactory(),
             ],
             expansion_strats=[
-                [strat.CellInsertionFactory(maxreqlen=length)],
-                [strat.PatternPlacementFactory()],
+                [
+                    strat.CellInsertionFactory(maxreqlen=length),
+                    strat.PatternPlacementFactory(partial=partial),
+                ],
             ],
             name=name,
         )
 
     @classmethod
-    def insertion_point_placements(
-        cls, length: int = 1, partial: bool = False
-    ) -> "TileScopePack":
+    def insertion_point_placements(cls, partial: bool = False) -> "TileScopePack":
         name = "insertion_"
-        if length > 1:
-            name += "length_{}_".format(length)
         partial_str = "partial_" if partial else ""
         name += f"{partial_str}point_placements"
         return TileScopePack(
             initial_strats=[
                 strat.FactorFactory(),
-                strat.RequirementCorroborationFactory(),
-                strat.CellInsertionFactory(maxreqlen=length, ignore_parent=True),
+                strat.CellInsertionFactory(
+                    maxreqlen=1, ignore_parent=True, one_cell_only=True
+                ),
             ],
             ver_strats=[
                 strat.BasicVerificationStrategy(),
@@ -280,8 +368,7 @@ class TileScopePack(StrategyPack):
         return TileScopePack(
             initial_strats=[
                 strat.FactorFactory(),
-                strat.RequirementCorroborationFactory(),
-                strat.CellInsertionFactory(ignore_parent=True),
+                strat.CellInsertionFactory(ignore_parent=True, one_cell_only=True),
             ],
             ver_strats=[strat.BasicVerificationStrategy()],
             inferral_strats=[],
@@ -336,7 +423,9 @@ class TileScopePack(StrategyPack):
         pack = cls.row_and_col_placements(row_only, col_only, partial)
         pack.name = "insertion_" + pack.name
         pack = pack.add_initial(
-            strat.CellInsertionFactory(maxreqlen=1, ignore_parent=True)
+            strat.CellInsertionFactory(
+                maxreqlen=1, ignore_parent=True, one_cell_only=True
+            )
         )
         return pack
 
@@ -385,11 +474,13 @@ class TileScopePack(StrategyPack):
             "length_{}_".format(length) if length != 2 else "",
             "partial_" if partial else "",
         )
+
+        initial_strats: List[CSSstrategy] = [strat.FactorFactory()]
+        if length > 1:
+            initial_strats.append(strat.RequirementCorroborationFactory())
+
         return TileScopePack(
-            initial_strats=[
-                strat.FactorFactory(),
-                strat.RequirementCorroborationFactory(),
-            ],
+            initial_strats=initial_strats,
             ver_strats=[
                 strat.BasicVerificationStrategy(),
                 strat.InsertionEncodingVerificationStrategy(),
@@ -401,8 +492,60 @@ class TileScopePack(StrategyPack):
                 strat.ObstructionTransitivityFactory(),
             ],
             expansion_strats=[
-                [strat.RequirementInsertionFactory(maxreqlen=length)],
-                [strat.PatternPlacementFactory(partial=partial)],
+                [
+                    strat.RequirementInsertionFactory(maxreqlen=length),
+                    strat.PatternPlacementFactory(partial=partial),
+                ],
+            ],
+            name=name,
+        )
+
+    @classmethod
+    def point_and_row_and_col_placements(
+        cls,
+        length: int = 1,
+        row_only: bool = False,
+        col_only: bool = False,
+        partial: bool = False,
+    ) -> "TileScopePack":
+        if row_only and col_only:
+            raise ValueError("Can't be row and col only.")
+        place_row = not col_only
+        place_col = not row_only
+        both = place_col and place_row
+        name = "{}{}point_and_{}{}{}_placements".format(
+            "length_{}_".format(length) if length > 1 else "",
+            "partial_" if partial else "",
+            "row" if not col_only else "",
+            "_and_" if both else "",
+            "col" if not row_only else "",
+        )
+        rowcol_strat = strat.RowAndColumnPlacementFactory(
+            place_row=place_row, place_col=place_col, partial=partial
+        )
+
+        initial_strats: List[CSSstrategy] = [strat.FactorFactory()]
+        if length > 1:
+            initial_strats.append(strat.RequirementCorroborationFactory())
+
+        return TileScopePack(
+            initial_strats=initial_strats,
+            ver_strats=[
+                strat.BasicVerificationStrategy(),
+                strat.InsertionEncodingVerificationStrategy(),
+                strat.OneByOneVerificationStrategy(),
+                strat.LocallyFactorableVerificationStrategy(),
+            ],
+            inferral_strats=[
+                strat.RowColumnSeparationStrategy(),
+                strat.ObstructionTransitivityFactory(),
+            ],
+            expansion_strats=[
+                [
+                    strat.CellInsertionFactory(maxreqlen=length),
+                    strat.PatternPlacementFactory(partial=partial),
+                    rowcol_strat,
+                ],
             ],
             name=name,
         )
